@@ -127,16 +127,14 @@ bool Curl_meets_timecondition(struct Curl_easy *data, time_t timeofdoc)
   case CURL_TIMECOND_IFMODSINCE:
   default:
     if(timeofdoc <= data->set.timevalue) {
-      infof(data,
-            "The requested document is not new enough");
+      infof(data, "The requested document is not new enough");
       data->info.timecond = TRUE;
       return FALSE;
     }
     break;
   case CURL_TIMECOND_IFUNMODSINCE:
     if(timeofdoc >= data->set.timevalue) {
-      infof(data,
-            "The requested document is not old enough");
+      infof(data, "The requested document is not old enough");
       data->info.timecond = TRUE;
       return FALSE;
     }
@@ -254,8 +252,9 @@ static CURLcode sendrecv_dl(struct Curl_easy *data,
     if(bytestoread && Curl_rlimit_active(&data->progress.dl.rlimit)) {
       curl_off_t dl_avail = Curl_rlimit_avail(&data->progress.dl.rlimit,
                                               Curl_pgrs_now(data));
-      /* DEBUGF(infof(data, "dl_rlimit, available=%" FMT_OFF_T, dl_avail));
-       */
+#if 0
+      DEBUGF(infof(data, "dl_rlimit, available=%" FMT_OFF_T, dl_avail));
+#endif
       /* In case of rate limited downloads: if this loop already got
        * data and less than 16k is left in the limit, break out.
        * We want to stutter a bit to keep in the limit, but too small
@@ -278,7 +277,7 @@ static CURLcode sendrecv_dl(struct Curl_easy *data,
       if(data->req.download_done && data->req.no_body &&
          !data->req.resp_trailer) {
         DEBUGF(infof(data, "EAGAIN, download done, no trailer announced, "
-               "not waiting for EOS"));
+                     "not waiting for EOS"));
         blen = 0;
         /* continue as if we received the EOS */
       }
@@ -534,7 +533,7 @@ CURLcode Curl_pretransfer(struct Curl_easy *data)
      * different ports! */
     data->state.allow_port = TRUE;
 
-#if defined(HAVE_SIGNAL) && defined(SIGPIPE) && !defined(HAVE_MSG_NOSIGNAL)
+#if defined(HAVE_SIGNAL) && defined(SIGPIPE) && !defined(MSG_NOSIGNAL)
     /*************************************************************
      * Tell signal handler to ignore SIGPIPE
      *************************************************************/
